"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const stripAnsi = require("strip-ansi");
const lodash = require("lodash");
const cli_framework_1 = require("@ionic/cli-framework");
const array_1 = require("@ionic/cli-framework/utils/array");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const string_1 = require("@ionic/cli-framework/utils/string");
const help_1 = require("@ionic/cli-utils/lib/help");
const ionic_1 = require("ionic");
const utils_1 = require("./utils");
const PROJECTS_DIRECTORY = path.resolve(__dirname, '..', '..', 'projects');
const STAGING_DIRECTORY = path.resolve(__dirname, '..', '..', '..', '..', '..', 'docs');
class DocsCommand extends cli_framework_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return {
                name: 'docs',
                summary: '',
            };
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield fs_1.removeDirectory(STAGING_DIRECTORY);
            yield fs_1.fsMkdirp(STAGING_DIRECTORY);
            const projectTypes = ['ionic-angular', 'ionic1']; // TODO: add 'angular'
            const baseCtx = yield ionic_1.generateContext();
            for (const projectType of projectTypes) {
                const projectJson = {
                    type: projectType,
                    commands: [],
                };
                // TODO: possible to do this without a physical directory?
                const ctx = Object.assign({}, baseCtx, { execPath: path.resolve(PROJECTS_DIRECTORY, projectType) });
                const executor = yield ionic_1.loadExecutor(ctx, [], {});
                const env = executor.namespace.env;
                const commands = yield this.getCommandList(executor.namespace);
                projectJson.commands = yield Promise.all(commands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.extractCommand(cmd); })));
                projectJson.commands.sort((a, b) => string_1.strcmp(a.name, b.name));
                yield fs_1.fsWriteFile(path.resolve(STAGING_DIRECTORY, `${projectType}.json`), JSON.stringify(projectJson, undefined, 2), { encoding: 'utf8' });
                env.close();
            }
            process.stdout.write(`${chalk_1.default.green('Done.')}\n`);
        });
    }
    extractCommand(command) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const commandPath = ['ionic', ...command.path.map(([p]) => p)];
            const namespacePath = lodash.initial(commandPath);
            const name = commandPath.join(' ');
            const summary = command.summary ? stripAnsi(utils_1.links2md(utils_1.ansi2md(command.summary))).trim() : '';
            const description = command.description ? stripAnsi(utils_1.links2md(utils_1.ansi2md(command.description))).trim() : '';
            const type = command.type;
            const exampleCommands = command.exampleCommands ? command.exampleCommands.map(c => `${name} ${c}`) : [];
            const aliases = command.aliases.map(a => [...namespacePath, a].join(' '));
            const filteredOptions = yield array_1.filter(command.options ? command.options : [], (opt) => tslib_1.__awaiter(this, void 0, void 0, function* () { return help_1.isOptionHidden(opt); }));
            const options = yield Promise.all(filteredOptions.map((opt) => tslib_1.__awaiter(this, void 0, void 0, function* () { return this.extractOption(opt); })));
            return { name, namespace: namespacePath, summary, description, type, exampleCommands, aliases, options };
        });
    }
    extractOption(option) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const name = option.name;
            const summary = option.summary ? stripAnsi(utils_1.links2md(utils_1.ansi2md(option.summary))).trim() : '';
            const aliases = option.aliases ? option.aliases : [];
            return { name, summary, default: option.default, aliases };
        });
    }
    getCommandList(namespace) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return array_1.filter(yield namespace.getCommandMetadataList(), (cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () { return help_1.isCommandHidden(cmd); }));
        });
    }
}
exports.DocsCommand = DocsCommand;
